//--------------------------------------------------------------------------
// Copyright (C) 2016-2019 Cisco and/or its affiliates. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License Version 2 as published
// by the Free Software Foundation.  You may not use, modify or distribute
// this program under any other version of the GNU General Public License.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//--------------------------------------------------------------------------

// host_cache_allocator.cc author Silviu Minut <sminut@cisco.com>

#ifndef HOST_CACHE_ALLOCATOR_CC
#define HOST_CACHE_ALLOCATOR_CC

#include "host_cache.h"

template <class T>
T* HostCacheAlloc<T>::allocate(std::size_t n)
{
    size_t sz=n*sizeof(T);
    T* out=std::allocator<T>::allocate(n);
    lru->update(sz);
    return out;
}

template <class T>
void HostCacheAlloc<T>::deallocate(T* p, std::size_t n) noexcept
{
    size_t sz = n*sizeof(T);
    std::allocator<T>::deallocate(p, n);
    lru->update( -(int) sz);
}

template <class T>
HostCacheAllocIp<T>::HostCacheAllocIp()
{
    lru = &host_cache;
}

#endif
